@@ -14,7 +14,7 @@ static R_CallMethodDef CallEntries[] = {
     {"mer_MCMCsamp", (DL_FUNC) &mer_MCMCsamp, 5},
     {"mer_coef", (DL_FUNC) &mer_coef, 2},
     {"mer_coefGets", (DL_FUNC) &mer_coefGets, 3},
-    {"mer_create", (DL_FUNC) &mer_create, 10},
+    {"mer_create", (DL_FUNC) &mer_create, 7},
     {"mer_dtCMatrix", (DL_FUNC) &mer_dtCMatrix, 1},
     {"mer_dtCMatrix_inv", (DL_FUNC) &mer_dtCMatrix_inv, 1},
     {"mer_factor", (DL_FUNC) &mer_factor, 1},
@@ -34,7 +34,10 @@ static R_CallMethodDef CallEntries[] = {
     {"mer_update_ZXy", (DL_FUNC) &mer_update_ZXy, 1},
     {"mer_update_y", (DL_FUNC) &mer_update_y, 2},
     {"pedigree_chol", (DL_FUNC) &pedigree_chol, 2},
-    {"Zt_create", (DL_FUNC) &Zt_create, 2},
+    {"Zt_carryOver", (DL_FUNC) &Zt_carryOver, 2},
+/*     {"Zt_create", (DL_FUNC) &Zt_create, 2}, */
+/*     {"Zt_create1", (DL_FUNC) &Zt_create1, 2}, */
+    {"Ztl_sparse", (DL_FUNC) &Ztl_sparse, 2},
     {NULL, NULL, 0}
 };
 
@@ -68,18 +71,15 @@ void R_init_lme4(DllInfo *dll)
     lme4_ZtZSym = install("ZtZ");
     lme4_ZtySym = install("Zty");
     lme4_bVarSym = install("bVar");
-    lme4_callSym = install("call");
     lme4_cnamesSym = install("cnames");
     lme4_devCompSym = install("devComp");
     lme4_devianceSym = install("deviance");
     lme4_diagSym = install("diag");
     lme4_factorSym = install("factor");
-    lme4_familySym = install("family");
     lme4_fixefSym = install("fixef");
     lme4_flistSym = install("flist");
     lme4_gradCompSym = install("gradComp");
     lme4_iSym = install("i");
-    lme4_methodSym = install("method");
     lme4_ncSym = install("nc");
     lme4_pSym = install("p");
     lme4_permSym = install("perm");
@@ -88,7 +88,6 @@ void R_init_lme4(DllInfo *dll)
     lme4_ranefSym = install("ranef");
     lme4_statusSym = install("status");
     lme4_uploSym = install("uplo");
-    lme4_useScaleSym = install("useScale");
     lme4_wrkresSym = install("wrkres");
     lme4_wtsSym = install("wts");
     lme4_xSym = install("x");
